/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* VOID:ENUM,ENUM,POINTER (../bijiben/src/libbiji/biji-marshalers.list:1) */
extern
void _biji_marshal_VOID__ENUM_ENUM_POINTER (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
                                            const GValue *param_values,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);

/* VOID:BOOLEAN,BOOLEAN (../bijiben/src/libbiji/biji-marshalers.list:2) */
extern
void _biji_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
                                          const GValue *param_values,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);

/* VOID:POINTER,ENUM (../bijiben/src/libbiji/biji-marshalers.list:3) */
extern
void _biji_marshal_VOID__POINTER_ENUM (GClosure     *closure,
                                       GValue       *return_value,
                                       guint         n_param_values,
                                       const GValue *param_values,
                                       gpointer      invocation_hint,
                                       gpointer      marshal_data);


G_END_DECLS
